import base64
import json
import threading
import logging
import http.server
from functools import partial
from urllib.parse import urlparse, parse_qsl

from openswitcher_proxy.frontend import AuthRequestHandler
from pyatem.field import FieldBase
import pyatem.command as commandmodule


class FieldEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, FieldBase):
            temp = obj.__dict__
            result = {}
            for key in temp:
                if key == 'raw':
                    continue
                result[key] = temp[key]
            return result
        elif isinstance(obj, bytes):
            return base64.b64encode(obj).decode()
        return obj


class ApiRequestHandler(AuthRequestHandler):
    def __init__(self, config, threadpool, *args, **kwargs):
        self.config = config
        self.threadpool = threadpool
        super().__init__(*args, **kwargs)

    def response(self, data, status=200):
        self.send_response(status)
        self.send_header('Content-type', 'application/json')
        self.end_headers()
        raw = json.dumps(data, cls=FieldEncoder, indent=2)
        self.wfile.write(raw.encode())

    def do_GET(self):
        if not self.verify_auth():
            return
        path = self.path
        allowed_hw = self.config['hardware'].split(',')
        if path == '/':
            hardware = []
            for hw in allowed_hw:
                row = {
                    'id': hw,
                }
                if hw not in self.threadpool['hardware']:
                    row['status'] = 'missing'
                else:
                    row['status'] = self.threadpool['hardware'][hw].get_status()
                hardware.append(row)
            self.response({'hardware': hardware})
            return
        parts = urlparse(path)
        path = parts.path[1:]
        part = path.split('/')
        args = parts.query
        if len(part) < 2:
            return self.response({}, 400)
        if part[0] not in allowed_hw:
            return self.response({'error': 'unknown device specified'}, 404)

        hw = part[0]
        fieldname = part[1]
        if fieldname in self.threadpool['hardware'][hw].switcher.mixerstate:
            field = self.threadpool['hardware'][hw].switcher.mixerstate[fieldname]
            return self.response(field)
        else:
            return self.response({'error': 'unknown field'}, 404)

    def do_POST(self):
        if not self.verify_auth():
            return
        path = self.path
        allowed_hw = self.config['hardware'].split(',')
        parts = urlparse(path)
        path = parts.path[1:]
        part = path.split('/')
        args = parts.query
        if len(part) < 2:
            return self.response({}, 400)
        if part[0] not in allowed_hw:
            return self.response({'error': 'unknown device specified'}, 404)

        hw = part[0]
        fieldname = part[1]
        classname = fieldname.title().replace('-', '') + "Command"
        if not hasattr(commandmodule, classname):
            return self.response({'error': 'unknown command'}, 404)

        rt = self.headers['Content-type']
        if rt is None:
            arguments = dict(parse_qsl(args))
        elif rt == 'application/x-www-form-urlencoded':
            length = int(self.headers['Content-length'])
            raw = self.rfile.read(length).decode()
            arguments = dict(parse_qsl(raw))
        elif rt == 'application/json':
            length = int(self.headers['Content-length'])
            raw = self.rfile.read(length).decode()
            arguments = json.loads(raw)
        else:
            return self.response({'error': 'unknown content-type'}, 400)

        for key in arguments:
            try:
                arguments[key] = int(arguments[key])
            except:
                pass
        if 'source' in arguments:
            inputs = self.threadpool['hardware'][hw].switcher.inputs
            if arguments['source'] in inputs:
                arguments['source'] = inputs[arguments['source']]

        try:
            cmd = getattr(commandmodule, classname)(**arguments)
            self.threadpool['hardware'][hw].switcher.send_commands([cmd])
        except Exception as e:
            return self.response({"error": str(e)}, 500)
        return self.response({"status": "ok"})


class HttpApiFrontendThread(threading.Thread):
    def __init__(self, config, threadlist):
        threading.Thread.__init__(self)
        self.name = 'http-api.' + str(config['bind'])
        self.config = config
        self.threadlist = threadlist
        self.stop = False
        self.server = None

    def run(self):
        logging.info('HTTP-Api frontend run')
        host, port = self.config['bind'].split(':')
        address = (host, int(port))
        logging.info(f'binding to {address}')

        handler = partial(ApiRequestHandler, self.config, self.threadlist)

        self.server = http.server.HTTPServer(address, handler)
        with self.server:
            self.server.serve_forever()

    def handler(self, *args):
        print(args)

    def get_status(self):
        return 'running'



import argparse
from pyatem.command import CutCommand, PreviewInputCommand
from pyatem.protocol import AtemProtocol

switcher = None
_inputs = {}
_switch_to = None
looping = True


def input_changed(inputproperties):
    global _inputs
    _inputs[inputproperties.short_name] = inputproperties.index


def connection_ready(*args):
    global looping
    select = PreviewInputCommand(index=0, source=_inputs[_switch_to])
    switch = CutCommand(index=0)
    switcher.send_commands([select, switch])
    looping = False


def main():
    global switcher, _switch_to
    parser = argparse.ArgumentParser(description="Atem CLI")
    parser.add_argument('ip', help='Atem ip or "usb" for usb')
    parser.add_argument('command', help='Command to execute')
    args = parser.parse_args()

    _switch_to = args.command

    if args.ip == 'usb':
        switcher = AtemProtocol(usb='auto')
    else:
        switcher = AtemProtocol(ip=args.ip)

    switcher.on('change:input-properties:*', input_changed)
    switcher.on('connected', connection_ready)

    switcher.connect()
    while looping:
        switcher.loop()


if __name__ == '__main__':
    main()



# Copyright 2021 - 2022, Martijn Braam and the OpenAtem contributors
# SPDX-License-Identifier: GPL-3.0-only
import colorsys
import logging

import cairo
import io
import math
from gi.repository import Gtk, GObject, Gio


class ColorWheelWidget(Gtk.DrawingArea):
    _gtype_name__ = 'ColorWheelWidget'

    red = GObject.Property(type=float, nick='Red value', default=0.2)
    green = GObject.Property(type=float, nick='Green value', default=0.1)
    blue = GObject.Property(type=float, nick='Blue value', default=0.1)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_size_request(100, 100)
        rainbow = Gio.resources_lookup_data('/nl/brixit/switcher/asset/rainbow.png', 0).get_data()
        rainbowpng = io.BytesIO(rainbow)
        self.rainbow = cairo.ImageSurface.create_from_png(rainbowpng)

    def get_coordinates(self):
        yiq = colorsys.rgb_to_yiq(self.red, self.green, self.blue)
        logging.debug(yiq)
        return yiq

    def set_coordinates(self, gain, x, y):
        rgb = colorsys.yiq_to_rgb(gain, x, y)
        self.red, self.green, self.blue = rgb

    def do_draw(self, cr):
        context = self.get_style_context()
        state = self.get_state_flags()
        context.set_state(state)

        size = self.get_allocation()

        # Draw background
        Gtk.render_background(context, cr, 0, 0, size.width, size.height)

        # Draw gain circle
        radius = (min(size.width, size.height) - 2) // 2
        cr.set_line_width(3.0)
        cr.arc(size.width // 2, size.height // 2, radius, 0, math.pi * 2)
        cr.stroke()

        # Draw center cross
        radius = radius * 0.9
        cr.set_line_width(1.0)
        cr.move_to((size.width // 2) - radius, size.height // 2)
        cr.line_to((size.width // 2) + radius, size.height // 2)
        cr.stroke()
        cr.move_to(size.width // 2, (size.height // 2) - radius)
        cr.line_to(size.width // 2, (size.height // 2) + radius)
        cr.stroke()

        # Draw color rainbow circle
        pattern_width = self.rainbow.get_width()
        if size.width > size.height:
            xoffset = (size.width - size.height) // 2
            yoffset = 0
            scale = size.height / pattern_width
        else:
            xoffset = 0
            yoffset = (size.height - size.width) // 2
            scale = size.width / pattern_width
        cr.set_source_surface(self.rainbow, xoffset / scale, yoffset / scale)
        source = cr.get_source()
        matrix = source.get_matrix()
        matrix.xx = 1 / scale
        matrix.yy = 1 / scale
        source.set_matrix(matrix)
        cr.set_line_width(16.0)
        cr.arc(size.width // 2, size.height // 2, radius, 0, math.pi * 2)
        cr.stroke()

        # Draw cursor
        gain, x, y = self.get_coordinates()
        cr.set_source_rgba(0, 0, 0, 1)
        cr.set_line_width(3)
        cr.arc((x * radius) + size.width // 2, (y * radius) + size.height // 2, 6, 0, math.pi * 2)
        cr.stroke()

        cr.save()
